# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

import functools

from c7n.query import QueryResourceManager, ChildResourceManager
from c7n.manager import resources
from c7n.utils import chunks, get_retry, generate_arn, local_session

from c7n.resources.shield import IsShieldProtected, SetShieldProtection


class Route53Base(object):

    permissions = ('route53:ListTagsForResources',)
    retry = staticmethod(get_retry(('Throttled',)))

    @property
    def generate_arn(self):
        if self._generate_arn is None:
            self._generate_arn = functools.partial(
                generate_arn,
                self.get_model().service,
                resource_type=self.get_model().type)
        return self._generate_arn

    def get_arn(self, r):
        return self.generate_arn(r[self.get_model().id].split("/")[-1])

    def augment(self, resources):
        _describe_route53_tags(
            self.get_model(), resources, self.session_factory,
            self.executor_factory, self.retry)
        return resources


def _describe_route53_tags(
        model, resources, session_factory, executor_factory, retry):

    def process_tags(resources):
        client = local_session(session_factory).client('route53')
        resource_map = {}
        for r in resources:
            k = r[model.id]
            if "hostedzone" in k:
                k = k.split("/")[-1]
            resource_map[k] = r

        results = retry(
            client.list_tags_for_resources,
            ResourceType=model.type,
            ResourceIds=list(resource_map.keys()))

        for resource_tag_set in results['ResourceTagSets']:
            if ('ResourceId' in resource_tag_set and
                    resource_tag_set['ResourceId'] in resource_map):
                resource_map[resource_tag_set['ResourceId']]['Tags'] = resource_tag_set['Tags']

    with executor_factory(max_workers=2) as w:
        return list(w.map(process_tags, chunks(resources, 20)))


@resources.register('hostedzone')
class HostedZone(Route53Base, QueryResourceManager):

    class resource_type(object):
        service = 'route53'
        type = 'hostedzone'
        enum_spec = ('list_hosted_zones', 'HostedZones', None)
        # detail_spec = ('get_hosted_zone', 'Id', 'Id', None)
        id = 'Id'
        filter_name = None
        name = 'Name'
        date = None
        dimension = None
        universal_taggable = True

    def get_arns(self, resource_set):
        arns = []
        for r in resource_set:
            _id = r[self.get_model().id].split("/")[-1]
            arns.append(self.generate_arn(_id))
        return arns


HostedZone.filter_registry.register('shield-enabled', IsShieldProtected)
HostedZone.action_registry.register('set-shield', SetShieldProtection)


@resources.register('healthcheck')
class HealthCheck(Route53Base, QueryResourceManager):

    class resource_type(object):
        service = 'route53'
        type = 'healthcheck'
        enum_spec = ('list_health_checks', 'HealthChecks', None)
        name = id = 'Id'
        filter_name = None
        date = None
        dimension = None
        universal_taggable = True


@resources.register('rrset')
class ResourceRecordSet(ChildResourceManager):

    class resource_type(object):
        service = 'route53'
        type = 'rrset'
        parent_spec = ('hostedzone', 'HostedZoneId')
        enum_spec = ('list_resource_record_sets', 'ResourceRecordSets', None)
        name = id = 'Name'
        filter_name = None
        date = None
        dimension = None


@resources.register('r53domain')
class Route53Domain(QueryResourceManager):

    class resource_type(object):
        service = 'route53domains'
        type = 'r53domain'
        enum_spec = ('list_domains', 'Domains', None)
        name = id = 'DomainName'
        filter_name = None
        date = None
        dimension = None



# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

import json
import os
import sys

from argparse import ArgumentTypeError
from c7n import cli, version, commands, utils
from datetime import datetime, timedelta

from .common import BaseTest, TextTestIO


class CliTest(BaseTest):
    """ A subclass of BaseTest with some handy functions for CLI related tests. """

    def patch_account_id(self):
        test_account_id = lambda x: self.account_id
        self.patch(cli, '_default_account_id', test_account_id)

    def get_output(self, argv):
        """ Run cli.main with the supplied argv and return the output. """
        out, err = self.run_and_expect_success(argv)
        return out

    def capture_output(self):
        out = TextTestIO()
        err = TextTestIO()
        self.patch(sys, 'stdout', out)
        self.patch(sys, 'stderr', err)
        return out, err

    def run_and_expect_success(self, argv):
        """ Run cli.main() with supplied argv and expect normal execution. """
        self.patch_account_id()
        self.patch(sys, 'argv', argv)
        out, err = self.capture_output()
        try:
            cli.main()
        except SystemExit as e:
            self.fail('Expected sys.exit would not be called. Exit code was ({})'.format(e.code))
        return out.getvalue(), err.getvalue()

    def run_and_expect_failure(self, argv, exit_code):
        """ Run cli.main() with supplied argv and expect exit_code. """
        self.patch_account_id()
        self.patch(sys, 'argv', argv)
        out, err = self.capture_output()
        #clear_resources()
        with self.assertRaises(SystemExit) as cm:
            cli.main()
        self.assertEqual(cm.exception.code, exit_code)
        return out.getvalue(), err.getvalue()

    def run_and_expect_exception(self, argv, exception):
        """ Run cli.main() with supplied argv and expect supplied exception. """
        self.patch_account_id()
        self.patch(sys, 'argv', argv)
        #clear_resources()
        try:
            cli.main()
        except exception:
            return
        self.fail('Error: did not raise {}.'.format(exception))


class UtilsTest(BaseTest):

    def test_key_val_pair(self):
        self.assertRaises(
            ArgumentTypeError,
            cli._key_val_pair,
            'invalid option',
        )
        param = 'day=today'
        self.assertIs(cli._key_val_pair(param), param)


class VersionTest(CliTest):

    def test_version(self):
        output = self.get_output(['custodian', 'version'])
        self.assertEqual(output.strip(), version.version)

    def test_debug_version(self):
        output = self.get_output(['custodian', 'version', '--debug'])
        # Among other things, this should print sys.path
        self.assertIn(version.version, output)
        self.assertIn(sys.path[0], output)


class ValidateTest(CliTest):

    def test_validate(self):
        invalid_policies = {
            'policies':
            [{
                'name': 'foo',
                'resource': 's3',
                'filters': [{"tag:custodian_tagging": "not-null"}],
                'actions': [{'type': 'untag',
                             'tags': {'custodian_cleanup': 'yes'}}],
            }]
        }
        yaml_file = self.write_policy_file(invalid_policies)
        json_file = self.write_policy_file(invalid_policies, format='json')

        # YAML validation
        self.run_and_expect_exception(['custodian', 'validate', yaml_file], SystemExit)

        # JSON validation
        self.run_and_expect_failure(['custodian', 'validate', json_file], 1)

        # no config files given
        self.run_and_expect_failure(['custodian', 'validate'], 1)

        # nonexistent file given
        self.run_and_expect_exception(
            ['custodian', 'validate', 'fake.yaml'], ValueError)

        valid_policies = {
            'policies':
            [{
                'name': 'foo',
                'resource': 's3',
                'filters': [{"tag:custodian_tagging": "not-null"}],
                'actions': [{'type': 'tag',
                             'tags': {'custodian_cleanup': 'yes'}}],
            }]
        }
        yaml_file = self.write_policy_file(valid_policies)

        self.run_and_expect_success(['custodian', 'validate', yaml_file])

        # legacy -c option
        self.run_and_expect_success(['custodian', 'validate', '-c', yaml_file])

        # duplicate policy names
        self.run_and_expect_failure(
            ['custodian', 'validate', yaml_file, yaml_file], 1)


class SchemaTest(CliTest):

    def test_schema(self):

        # no options
        self.run_and_expect_success(['custodian', 'schema'])

        # summary option
        self.run_and_expect_success(['custodian', 'schema', '--summary'])

        # json option
        self.run_and_expect_success(['custodian', 'schema', '--json'])

        # with just a resource
        self.run_and_expect_success(['custodian', 'schema', 'ec2'])

        # resource.actions
        self.run_and_expect_success(['custodian', 'schema', 'ec2.actions'])

        # resource.filters
        self.run_and_expect_success(['custodian', 'schema', 'ec2.filters'])

        # specific item
        self.run_and_expect_success(
            ['custodian', 'schema', 'ec2.filters.tag-count'])

    def test_invalid_options(self):

        # invalid resource
        self.run_and_expect_failure(['custodian', 'schema', 'fakeresource'], 1)

        # invalid category
        self.run_and_expect_failure(
            ['custodian', 'schema', 'ec2.arglbargle'], 1)

        # invalid item
        self.run_and_expect_failure(
            ['custodian', 'schema', 'ec2.filters.nonexistent'], 1)

        # invalid number of selectors
        self.run_and_expect_failure(
            ['custodian', 'schema', 'ec2.filters.and.foo'], 1)

    def test_schema_output(self):

        output = self.get_output(['custodian', 'schema'])
        self.assertIn('ec2', output)

        output = self.get_output(['custodian', 'schema', 'ec2'])
        self.assertIn('actions:', output)
        self.assertIn('filters:', output)

        output = self.get_output(['custodian', 'schema', 'ec2.filters'])
        self.assertNotIn('actions:', output)
        self.assertIn('filters:', output)

        output = self.get_output(['custodian', 'schema', 'ec2.filters.image'])
        self.assertIn('Help', output)


class ReportTest(CliTest):

    def test_report(self):
        policy_name = 'ec2-running-instances'
        valid_policies = {
            'policies':
            [{
                'name': policy_name,
                'resource': 'ec2',
                'query': [{"instance-state-name": "running"}],
            }]
        }
        yaml_file = self.write_policy_file(valid_policies)

        output = self.get_output(
            ['custodian', 'report', '-s', self.output_dir, yaml_file])
        self.assertIn('InstanceId', output)
        self.assertIn('i-014296505597bf519', output)

        # ASCII formatted test
        output = self.get_output(
            ['custodian', 'report', '--format', 'grid', '-s', self.output_dir, yaml_file])
        self.assertIn('InstanceId', output)
        self.assertIn('i-014296505597bf519', output)

        # empty file
        temp_dir = self.get_temp_dir()
        empty_policies = {'policies': []}
        yaml_file = self.write_policy_file(empty_policies)
        self.run_and_expect_failure(
            ['custodian', 'report', '-s', temp_dir, yaml_file],
            1)

        # more than 1 policy
        policies = {
            'policies': [
                {'name': 'foo', 'resource': 's3'},
                {'name': 'bar', 'resource': 'ec2'},
            ]
        }
        yaml_file = self.write_policy_file(policies)
        self.run_and_expect_failure(
            ['custodian', 'report', '-s', temp_dir, yaml_file],
            1)

    def test_warning_on_empty_policy_filter(self):
        # This test is to examine the warning output supplied when -p is used and
        # the resulting policy set is empty.  It is not specific to the `report`
        # subcommand - it is also used by `run` and a few other subcommands.

        policy_name = 'test-policy'
        valid_policies = {
            'policies':
            [{
                'name': policy_name,
                'resource': 's3',
                'filters': [{"tag:custodian_tagging": "not-null"}],
            }]
        }
        yaml_file = self.write_policy_file(valid_policies)
        temp_dir = self.get_temp_dir()

        bad_policy_name = policy_name + '-nonexistent'
        log_output = self.capture_logging('custodian.commands')
        self.run_and_expect_failure(
            ['custodian', 'report', '-s', temp_dir, '-p', bad_policy_name, yaml_file],
            1)
        self.assertIn(policy_name, log_output.getvalue())

        bad_resource_name = 'foo'
        self.run_and_expect_failure(
            ['custodian', 'report', '-s', temp_dir, '-t', bad_resource_name, yaml_file],
            1)


class LogsTest(CliTest):

    def test_logs(self):
        temp_dir = self.get_temp_dir()

        # Test 1 - empty file
        empty_policies = {'policies': []}
        yaml_file = self.write_policy_file(empty_policies)
        self.run_and_expect_failure(
            ['custodian', 'logs', '-s', temp_dir, yaml_file],
            1)

        # Test 2 - more than one policy
        policies = {
            'policies': [
                {'name': 'foo', 'resource': 's3'},
                { 'name': 'bar', 'resource': 'ec2'},
            ]
        }
        yaml_file = self.write_policy_file(policies)
        self.run_and_expect_failure(
            ['custodian', 'logs', '-s', temp_dir, yaml_file],
            1)

        # Test 3 - successful test
        p_data = {
            'name': 'test-policy',
            'resource': 'rds',
            'filters': [
                {
                    'key': 'GroupName',
                    'type': 'security-group',
                    'value': 'default',
                },
            ],
            'actions': [{'days': 10, 'type': 'retention'}],
        }
        yaml_file = self.write_policy_file({'policies': [p_data]})
        output_dir = os.path.join(
            os.path.dirname(__file__),
            'data',
            'logs',
        )
        self.run_and_expect_success(
            ['custodian', 'logs', '-s', output_dir, yaml_file],
        )


class TabCompletionTest(CliTest):
    """ Tests for argcomplete tab completion. """

    def test_schema_completer(self):
        self.assertIn('rds', cli.schema_completer('rd'))
        self.assertIn('s3.', cli.schema_completer('s3'))
        self.assertListEqual([], cli.schema_completer('invalidResource.'))
        self.assertIn('rds.actions', cli.schema_completer('rds.'))
        self.assertIn('s3.filters.', cli.schema_completer('s3.filters'))
        self.assertIn('s3.filters.event', cli.schema_completer('s3.filters.eve'))
        self.assertListEqual([], cli.schema_completer('rds.actions.foo.bar'))

    def test_schema_completer_wrapper(self):
        class MockArgs(object):
            summary = False

        args = MockArgs()
        self.assertIn('rds', cli._schema_tab_completer('rd', args))

        args.summary = True
        self.assertListEqual([], cli._schema_tab_completer('rd', args))


class RunTest(CliTest):

    def test_ec2(self):
        session_factory = self.replay_flight_data(
            'test_ec2_state_transition_age_filter'
        )

        from c7n.policy import PolicyCollection
        self.patch(
            PolicyCollection, 'session_factory',
            staticmethod(lambda x=None: session_factory))

        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file({
            'policies': [{
                'name': 'ec2-state-transition-age',
                'resource': 'ec2',
                'filters': [
                    {'State.Name': 'running'},
                    {'type': 'state-age', 'days': 30},
                ]
            }]
        })

        # TODO - capture logging and ensure the following
        #self.assertIn('Running policy ec2-state-transition-age', logs)
        #self.assertIn('metric:ResourceCount Count:1 policy:ec2-state-transition-age', logs)

        self.run_and_expect_success(
            [
                'custodian', 'run',
                '--cache', temp_dir + '/cache',
                '-s', temp_dir,
                yaml_file,
            ],
        )

    def test_error(self):
        from c7n.policy import Policy
        self.patch(Policy, '__call__', lambda x: (_ for _ in ()).throw(Exception('foobar')))

        #
        # Make sure that if the policy causes an exception we error out
        #

        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file({
            'policies': [{
                'name': 'error',
                'resource': 'ec2',
                'filters': [
                    {'State.Name': 'running'},
                    {'type': 'state-age', 'days': 30},
                ],
            }]
        })

        self.run_and_expect_failure(
            [
                'custodian', 'run',
                '--cache', temp_dir + '/cache',
                '-s', temp_dir,
                yaml_file,
            ],
            2
        )

        #
        # Test --debug
        #
        class CustomError(Exception):
            pass

        import pdb
        self.patch(pdb, 'post_mortem', lambda x: (_ for _ in ()).throw(CustomError))

        self.run_and_expect_exception(
            ['custodian', 'run', '-s', temp_dir, '--debug', yaml_file],
            CustomError
        )


class MetricsTest(CliTest):

    def test_metrics(self):
        session_factory = self.replay_flight_data('test_lambda_policy_metrics')
        from c7n.policy import PolicyCollection

        self.patch(
            PolicyCollection, 'session_factory',
            staticmethod(lambda x=None: session_factory))

        yaml_file = self.write_policy_file({
            'policies': [{
                'name': 'ec2-tag-compliance-v6',
                'resource': 'ec2',
                'mode': {
                    'type': 'ec2-instance-state',
                    'events': ['running']},
                'filters': [
                    {"tag:custodian_status": 'absent'},
                    {'or': [
                        {"tag:App": 'absent'},
                        {"tag:Env": 'absent'},
                        {"tag:Owner": 'absent'}]}]
            }]
        })

        end = datetime.utcnow()
        start = end - timedelta(14)
        period = 24 * 60 * 60 * 14

        out = self.get_output(
            ['custodian', 'metrics', '--start', str(start), '--end', str(end), '--period', str(period), yaml_file])

        self.assertEqual(
            json.loads(out),
            {'ec2-tag-compliance-v6':
             {u'Durations': [],
              u'Errors': [{u'Sum': 0.0,
                           u'Timestamp': u'2016-05-30T10:50:00+00:00',
                           u'Unit': u'Count'}],
              u'Invocations': [{u'Sum': 4.0,
                                u'Timestamp': u'2016-05-30T10:50:00+00:00',
                                u'Unit': u'Count'}],
              u'ResourceCount': [{u'Average': 1.0,
                                  u'Sum': 2.0,
                                  u'Timestamp': u'2016-05-30T10:50:00+00:00',
                                  u'Unit': u'Count'}],
              u'Throttles': [{u'Sum': 0.0,
                              u'Timestamp': u'2016-05-30T10:50:00+00:00',
                              u'Unit': u'Count'}]}})

    def test_metrics_get_endpoints(self):

        #
        # Test for defaults when --start is not supplied
        #
        class FakeOptions(object):
            start = end = None
            days = 5
        options = FakeOptions()
        start, end = commands._metrics_get_endpoints(options)
        self.assertEqual((end - start).days, options.days)

        #
        # Test that --start and --end have to be passed together
        #
        policy = {
            'policies':
            [{
                'name': 'metrics-test',
                'resource': 'ec2',
                'query': [{"instance-state-name": "running"}],
            }]
        }
        yaml_file = self.write_policy_file(policy)

        self.run_and_expect_failure(
            ['custodian', 'metrics', '--start', '1', yaml_file],
            1
        )


class MiscTest(CliTest):

    def test_empty_policy_file(self):
        # Doesn't do anything, but should exit 0
        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file({})
        self.run_and_expect_success(
            ['custodian', 'run', '-s', temp_dir, yaml_file]
        )

    def test_nonexistent_policy_file(self):
        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file({})
        nonexistent = yaml_file + '.bad'
        self.run_and_expect_failure(
            ['custodian', 'run', '-s', temp_dir, yaml_file, nonexistent],
            1)

    def test_duplicate_policy(self):
        policy = {
            'policies':
            [{
                'name': 'metrics-test',
                'resource': 'ec2',
                'query': [{"instance-state-name": "running"}],
            }]
        }
        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file(policy)
        self.run_and_expect_failure(
            ['custodian', 'run', '-s', temp_dir, yaml_file, yaml_file],
            1)

    def test_failure_with_no_default_region(self):
        policy = {
            'policies':
            [{
                'name': 'will-never-run',
                'resource': 'ec2',
            }]
        }
        temp_dir = self.get_temp_dir()
        yaml_file = self.write_policy_file(policy)
        self.patch(utils, 'get_profile_session', lambda x: None)
        self.run_and_expect_failure(
            ['custodian', 'run', '-s', temp_dir, yaml_file],
            1)



# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

from botocore.exceptions import ClientError

import json
from c7n.filters import CrossAccountAccessFilter
from c7n.manager import resources
from c7n.query import QueryResourceManager
from c7n.actions import RemovePolicyBase
from c7n.utils import local_session


@resources.register('ecr')
class ECR(QueryResourceManager):

    class resource_type(object):
        service = 'ecr'
        enum_spec = ('describe_repositories', 'repositories', None)
        name = "repositoryName"
        id = "repositoryArn"
        dimension = None
        filter_name = 'repositoryNames'
        filter_type = 'list'


ErrPolicyNotFound = 'RepositoryPolicyNotFoundException'


@ECR.filter_registry.register('cross-account')
class ECRCrossAccountAccessFilter(CrossAccountAccessFilter):
    """Filters all EC2 Container Registries (ECR) with cross-account access

    :example:

        .. code-block: yaml

            policies:
              - name: ecr-cross-account
                resource: ecr
                filters:
                  - type: cross-account
                    whitelist_from:
                      expr: "accounts.*.accountNumber"
                      url: *accounts_url
    """
    permissions = ('ecr:GetRepositoryPolicy',)

    def process(self, resources, event=None):

        def _augment(r):
            client = local_session(self.manager.session_factory).client('ecr')
            try:
                r['Policy'] = client.get_repository_policy(
                    repositoryName=r['repositoryName'])['policyText']
            except ClientError as e:
                if e.response['Error']['Code'] == ErrPolicyNotFound:
                    return None
                raise
            return r

        self.log.debug("fetching policy for %d repos" % len(resources))
        with self.executor_factory(max_workers=3) as w:
            resources = list(filter(None, w.map(_augment, resources)))

        return super(ECRCrossAccountAccessFilter, self).process(
            resources, event)


@ECR.action_registry.register('remove-statements')
class RemovePolicyStatement(RemovePolicyBase):
    """Action to remove policy statements from ECR

    :example:

        .. code-block: yaml

            policies:
              - name: ecr-remove-cross-accounts
                resource: ecr
                filters:
                  - type: cross-account
                actions:
                  - type: remove-statements
                    statement_ids: matched
    """

    permissions = ("ecr:SetRepositoryPolicy", "ecr:GetRepositoryPolicy")

    def process(self, resources):
        results = []
        client = local_session(self.manager.session_factory).client('ecr')
        for r in resources:
            try:
                results += filter(None, [self.process_resource(client, r)])
            except Exception:
                self.log.exception(
                    "Error processing ecr registry:%s", r['repositoryArn'])
        return results

    def process_resource(self, client, resource):
        if 'Policy' not in resource:
            try:
                resource['Policy'] = client.get_repository_policy(
                    repositoryName=resource['repositoryName'])['policyText']
            except ClientError as e:
                if e.response['Error']['Code'] != ErrPolicyNotFound:
                    raise
                resource['Policy'] = None

        if not resource['Policy']:
            return

        p = json.loads(resource['Policy'])
        statements, found = self.process_policy(
            p, resource, CrossAccountAccessFilter.annotation_key)

        if not found:
            return

        if not statements:
            client.delete_repository_policy(
                repositoryName=resource['repositoryName'])
        else:
            client.set_repository_policy(
                repositoryName=resource['repositoryName'],
                policyText=json.dumps(p))
        return {'Name': resource['repositoryName'],
                'State': 'PolicyRemoved',
                'Statements': found}



# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

import io
import json
import os
import tempfile
import mock

from unittest import TestCase

from c7n.resources import s3
from c7n.ufuncs import s3crypt
from .common import BaseTest
from six.moves.urllib_parse import quote_plus

class TestS3Crypt(TestCase):

    def setUp(self):
        self.old_dir = os.getcwd()
        os.chdir(tempfile.gettempdir())
        self.config_data = {
            'test': 'data',
            'large': False,
        }
        with open('config.json', 'w') as conf:
            json.dump(self.config_data, conf)

    def tearDown(self):
        os.remove('config.json')
        os.chdir(self.old_dir)

    def test_init(self):
        s3crypt.init()
        self.assertEqual(s3crypt.config, self.config_data)

        # Run a second time to ensure it is idempotent
        s3crypt.init()
        self.assertEqual(s3crypt.config, self.config_data)


class TestS3CryptEvent(BaseTest):
    @mock.patch('sys.stdout', new_callable=io.StringIO)
    def test_s3_event_simple(self, mock_stdout):
        self.patch(s3, 'S3_AUGMENT_TABLE', [])
        session_factory = self.replay_flight_data('test_s3_encrypt')
        client = session_factory().client('s3')
        self.patch(s3crypt, 's3', client)

        event = {'Records': [{
            's3': {
                'bucket': {
                    'name': 'test-bucket'
                },
                'object': {
                    'key': quote_plus('test-key'),
                    'size': 42
                }
            }
        }]}
        s3crypt.process_event(event, {})

    @mock.patch('sys.stdout', new_callable=io.StringIO)
    def test_s3_event_unsafe_key(self, mock_stdout):
        self.patch(s3, 'S3_AUGMENT_TABLE', [])
        session_factory = self.replay_flight_data('test_s3_encrypt')
        client = session_factory().client('s3')
        self.patch(s3crypt, 's3', client)

        event = {'Records': [{
            's3': {
                'bucket': {
                    'name': 'test-bucket'
                },
                'object': {
                    'key': quote_plus('/test000/!-_.*\'()/&@:,$=+%2b?;/ /whatever'),
                    'size': 42
                }
            }
        }]}
        s3crypt.process_event(event, {})

    @mock.patch('sys.stdout', new_callable=io.StringIO)
    def test_s3_event_version(self, mock_stdout):
        self.patch(s3, 'S3_AUGMENT_TABLE', [])
        session_factory = self.replay_flight_data('test_s3_encrypt')
        client = session_factory().client('s3')
        self.patch(s3crypt, 's3', client)

        event = {'Records': [{
            's3': {
                'bucket': {
                    'name': 'test-bucket'
                },
                'object': {
                    'key': quote_plus('test-key'),
                    'size': 42,
                    'versionId': '99'
                }
            }
        }]}
        s3crypt.process_event(event, {})



# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from .common import BaseTest, Bag

from botocore.vendored import requests
from c7n.filters.locked import SignatureAuth, Locked

def noop(*args):
    return

class LockedTests(BaseTest):

    def test_auth(self):
        auth = SignatureAuth(
            Bag(secret_key='foo', access_key='bar', method='env', token=None),
            'us-east-1',
            'execute-api')

        req = requests.Request('POST', 'http://example.com', auth=auth)
        prepped = req.prepare()
        self.assertTrue(
            prepped.headers['Authorization'].startswith(
                'AWS4-HMAC-SHA256 Credential=bar/'))

    def test_unlocked(self):
        def get(*args, **kw):
            return {'LockStatus': 'unlocked'}
        self.patch(Locked, 'get_api_credentials', noop)
        self.patch(Locked, 'get_lock_status', get)
        p = self.load_policy({
            'name': 'ltest',
            'resource': 'security-group',
            'filters': [
                {'type': 'locked', 'endpoint': 'http://example.com/bar'}]})
        f_locked = p.resource_manager.filters[0]
        result = f_locked.process([
            {'GroupId': 'sg-123', 'VpcId': 'vpc-123'}])
        self.assertEqual(len(result), 0)

    def test_status_error(self):
        def get(*args, **kw):
            return {'Message': 'unknown'}
        self.patch(Locked, 'get_api_credentials', noop)
        self.patch(Locked, 'get_lock_status', get)
        p = self.load_policy({
            'name': 'ltest',
            'resource': 'security-group',
            'filters': [
                {'type': 'locked', 'endpoint': 'http://example.com/bar'}]})
        f_locked = p.resource_manager.filters[0]
        self.assertRaises(
            RuntimeError,
            f_locked.process,
            [{'GroupId': 'sg-123', 'VpcId': 'vpc-123'}])

    def test_locked(self):
        def get(*args, **kw):
            return {'LockStatus': 'locked', 'RevisionDate': time.time()}
        self.patch(Locked, 'get_api_credentials', noop)
        self.patch(Locked, 'get_lock_status', get)
        p = self.load_policy({
            'name': 'ltest',
            'resource': 'security-group',
            'filters': [
                {'type': 'locked', 'endpoint': 'http://example.com/bar'}]})
        f_locked = p.resource_manager.filters[0]
        result = f_locked.process([
            {'GroupId': 'sg-123', 'VpcId': 'vpc-123'}])
        self.assertEqual(len(result), 1)




# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

import jmespath
from .common import BaseTest
from c7n.utils import local_session


class CloudFrontWaf(BaseTest):

    def test_waf(self):
        factory = self.replay_flight_data('test_distribution_waf')
        p = self.load_policy({
            'name': 'waf-cfront',
            'resource': 'distribution',
            'filters': [{
                'type': 'waf-enabled',
                'web-acl': 'test',
                'state': False}],
            'actions': [
                {'type': 'set-waf',
                 'web-acl': 'test',
                 'state': True}]},
            session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)
            

        p = self.load_policy({
            'name': 'waf-cfront',
            'resource': 'distribution',
            'filters': [{
                'type': 'waf-enabled',
                'web-acl': 'test',
                'state': False}]},
            session_factory=factory)
        self.assertEqual(p.run(), [])


class CloudFront(BaseTest):

    def test_shield_metric_filter(self):
        factory = self.replay_flight_data('test_distribution_shield_metrics')
        p = self.load_policy({
            'name': 'ddos-filter',
            'resource': 'distribution',
            'filters': [{
                'type': 'shield-metrics',
                'name': 'DDoSDetected',
                'value': 1,
                'op': 'ge'}]},
            session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 0)

    def test_distribution_metric_filter(self):
        factory = self.replay_flight_data('test_distribution_metric_filter')
        p = self.load_policy({
            'name': 'requests-filter',
            'resource': 'distribution',
            'filters': [{
                'type': 'metrics',
                'name': 'Requests',
                'value': 3,
                'op': 'ge'
            }]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(
            resources[0]['DomainName'], 'd32plmcrnvwzrd.cloudfront.net')

    def test_distribution_set_ssl(self):
        factory = self.replay_flight_data('test_distrbution_set_ssl')

        k = 'DefaultCacheBehavior.ViewerProtocolPolicy'

        p = self.load_policy({
            'name': 'distribution-set-ssl',
            'resource': 'distribution',
            'filters': [{
                'type': 'value',
                'key': k,
                'value': 'allow-all',
                'op': 'contains'
            }],
            'actions': [{
                'type': 'set-protocols',
                'ViewerProtocolPolicy': 'https-only'
            }]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        expr = jmespath.compile(k)
        r = expr.search(resources[0])
        self.assertTrue('allow-all' in r)

        client = local_session(factory).client('cloudfront')
        resp = client.list_distributions()
        self.assertEqual(
            resp['DistributionList']['Items'][0]['DefaultCacheBehavior']['ViewerProtocolPolicy'],
            'https-only')

    def test_distribution_custom_origin(self):
        factory = self.replay_flight_data('test_distrbution_custom_origin')

        k = 'Origins.Items[].CustomOriginConfig.OriginSslProtocols.Items[]'

        p = self.load_policy({
            'name': 'distribution-set-ssl',
            'resource': 'distribution',
            'filters': [{
                'type': 'value',
                'key': k,
                'value': 'TLSv1',
                'op': 'contains'
            }],
            'actions': [{
                'type': 'set-protocols',
                'OriginSslProtocols': ['TLSv1.1','TLSv1.2'],
                'OriginProtocolPolicy': 'https-only'
            }]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        expr = jmespath.compile(k)
        r = expr.search(resources[0])
        self.assertTrue('TLSv1' in r)

        client = local_session(factory).client('cloudfront')
        resp = client.list_distributions()
        self.assertEqual(
            resp['DistributionList']['Items'][0]['Origins']['Items'][0]['CustomOriginConfig']['OriginProtocolPolicy'],
            'https-only')
        self.assertTrue('TLSv1.2' in
            resp['DistributionList']['Items'][0]['Origins']['Items'][0]['CustomOriginConfig']['OriginSslProtocols']['Items'])
        self.assertFalse('TLSv1' in
            resp['DistributionList']['Items'][0]['Origins']['Items'][0]['CustomOriginConfig']['OriginSslProtocols']['Items'])

    def test_distribution_disable(self):
        factory = self.replay_flight_data('test_distrbution_disable')

        p = self.load_policy({
            'name': 'distribution-disable',
            'resource': 'distribution',
            'filters': [{
                'type': 'value',
                'key': 'DefaultCacheBehavior.ViewerProtocolPolicy',
                'value': 'allow-all',
                'op': 'contains'
            }],
            'actions': [
                {
                    'type': 'disable'
                }
            ]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['Enabled'], True)

        client = local_session(factory).client('cloudfront')
        resp = client.list_distributions()
        self.assertEqual(resp['DistributionList']['Items'][0]['Enabled'], False)


    def test_distribution_tag(self):
        factory = self.replay_flight_data('test_distrbution_tag')

        p = self.load_policy({
            'name': 'distribution-tag',
            'resource': 'distribution',
            'filters': [{
                'tag:abc': 'present'
            }],
            'actions': [
                {
                    'type': 'tag',
                    'key': '123',
                    'value': '456'
                }
            ]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        client = local_session(factory).client('cloudfront')
        resp = client.list_tags_for_resource(
            Resource=resources[0]['ARN'])
        self.assertEqual(len(resp['Tags']['Items']), 2)


    def test_streaming_distribution_disable(self):
        factory = self.replay_flight_data('test_streaming_distrbution_disable')

        p = self.load_policy({
            'name': 'streaming-distribution-disable',
            'resource': 'streaming-distribution',
            'filters': [{
                'type': 'value',
                'key': 'S3Origin.OriginAccessIdentity',
                'value': ''
            }],
            'actions': [{
                'type': 'disable'
            }]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['Enabled'], True)

        client = local_session(factory).client('cloudfront')
        resp = client.list_streaming_distributions()
        self.assertEqual(resp['StreamingDistributionList']['Items'][0]['Enabled'], False)

    def test_streaming_distribution_tag(self):
        factory = self.replay_flight_data('test_streaming_distrbution_tag')

        p = self.load_policy({
            'name': 'streaming-distribution-tag',
            'resource': 'streaming-distribution',
            'filters': [{
                'tag:123': 'present'
            }],
            'actions': [
                {
                    'type': 'tag',
                    'key': 'abc',
                    'value': '123'
                }
            ]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        client = local_session(factory).client('cloudfront')
        resp = client.list_tags_for_resource(
            Resource=resources[0]['ARN'])
        self.assertEqual(len(resp['Tags']['Items']), 2)



# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

from c7n.executor import MainThreadExecutor
from c7n.resources.rdscluster import RDSCluster
from c7n import tags

from .common import BaseTest


class RDSClusterTest(BaseTest):

    def remove_augments(self):
        # This exists because we added tag augmentation after eight other tests
        # were created and I did not want to re-create the state to re-record
        # them with the extra API call. If those get re-recorded we can remove
        # this.
        self.patch(RDSCluster, 'augment', lambda x, y: y)

    def test_rdscluster_security_group(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_sg_filter')
        p = self.load_policy({
            'name': 'rdscluster-sg',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'security-group',
                 'key': 'GroupName',
                 'value': 'default'}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['DatabaseName'], 'devtest')

    def test_rdscluster_subnet(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_subnet')
        p = self.load_policy({
            'name': 'rdscluster-sub',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'subnet',
                 'key': 'MapPublicIpOnLaunch',
                 'value': True}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['DatabaseName'], 'devtest')

    def test_rdscluster_simple(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_simple')
        p = self.load_policy({
            'name': 'rdscluster-simple',
            'resource': 'rds-cluster'},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)

    def test_rdscluster_simple_filter(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_simple')
        p = self.load_policy({
            'name': 'rdscluster-simple-filter',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'value',
                 'key': 'DBClusterIdentifier',
                 'value': 'bbb'}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_delete(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_delete')
        p = self.load_policy({
            'name': 'rdscluster-delete',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'value',
                 'key': 'DBClusterIdentifier',
                 'value': 'bbb'}],
            'actions': [
                {'type': 'delete',
                 'delete-instances': False}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_delete_with_instances(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_delete_with_instances')
        p = self.load_policy({
            'name': 'rdscluster-delete',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'value',
                 'key': 'DBClusterIdentifier',
                 'value': 'bbb'}],
            'actions': [
                {'type': 'delete',
                 'delete-instances': True}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_retention(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_retention')
        p = self.load_policy({
            'name': 'rdscluster-delete',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'value',
                 'key': 'DBClusterIdentifier',
                 'value': 'bbb'}],
            'actions': [{'type': 'retention', 'days': 21}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_snapshot(self):
        self.remove_augments()
        session_factory = self.replay_flight_data('test_rdscluster_snapshot')
        p = self.load_policy({
            'name': 'rdscluster-snapshot',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'value',
                 'key': 'DBClusterIdentifier',
                 'value': 'bbb'}],
            'actions': [{'type': 'snapshot'}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_tag_augment(self):
        session_factory = self.replay_flight_data('test_rdscluster_tag_augment')
        p = self.load_policy({
            'name': 'rdscluster-tag-augment',
            'resource': 'rds-cluster',
            'filters': [{'tag:cfoo': 'cbar'}]},
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_tag_and_remove(self):
        self.patch(RDSCluster, 'executor_factory', MainThreadExecutor)
        session_factory = self.replay_flight_data(
            'test_rdscluster_tag_and_remove'
        )
        client = session_factory().client('rds')

        p = self.load_policy({
            'name': 'rds-cluster-tag',
            'resource': 'rds-cluster',
            'filters': [
                {'DBClusterIdentifier': 'c7ntest'}
            ],
            'actions': [
                {'type': 'tag', 'key': 'xyz', 'value': 'hello world'}
            ]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        arn = p.resource_manager.generate_arn(
            resources[0]['DBClusterIdentifier'])

        tags = client.list_tags_for_resource(ResourceName=arn)
        tag_map = {t['Key']: t['Value'] for t in tags['TagList']}
        self.assertTrue('xyz' in tag_map)

        policy = self.load_policy({
            'name': 'rds-cluster-remove-tag',
            'resource': 'rds-cluster',
            'filters': [
                {'tag:xyz': 'not-null'}
            ],
            'actions': [
                {'type': 'remove-tag', 'tags': ['xyz']}
            ]},
            session_factory=session_factory)
        resources = policy.run()
        self.assertEqual(len(resources), 1)

        tags = client.list_tags_for_resource(ResourceName=arn)
        tag_map = {t['Key']: t['Value'] for t in tags['TagList']}
        self.assertFalse('xyz' in tag_map)

    def test_rdscluster_mark_match_unmark(self):
        session_factory = self.replay_flight_data(
            'test_rdscluster_mark_and_match'
        )
        client = session_factory().client('rds')

        # mark
        p = self.load_policy({
            'name': 'rds-mark',
            'resource': 'rds-cluster',
            'filters': [
                {'DBClusterIdentifier': 'c7ntest'}],
            'actions': [
                {'type': 'mark-for-op', 'tag': 'custodian_next', 'days': 1,
                 'op': 'delete'}
            ]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        # assert marked
        arn = p.resource_manager.generate_arn(
            resources[0]['DBClusterIdentifier']
        )
        tags = client.list_tags_for_resource(ResourceName=arn)
        tag_map = {t['Key']: t['Value'] for t in tags['TagList']}
        self.assertTrue('custodian_next' in tag_map)

        # match marked
        policy = self.load_policy({
            'name': 'rds-mark-filter',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'marked-for-op', 'tag': 'custodian_next',
                 'op': 'delete', 'skew': 1}
            ]},
            session_factory=session_factory)
        resources = policy.run()
        self.assertEqual(len(resources), 1)

        # unmark
        policy = self.load_policy({
            'name': 'rds-mark-filter',
            'resource': 'rds-cluster',
            'filters': [
                {'type': 'marked-for-op', 'tag': 'custodian_next',
                 'op': 'delete', 'skew': 1}
            ],
            'actions': [
                {'type': 'unmark', 'tags': ['custodian_next']}
            ]},
            session_factory=session_factory)
        resources = policy.run()
        self.assertEqual(len(resources), 1)

        # assert unmarked
        tags = client.list_tags_for_resource(ResourceName=arn)
        tag_map = {t['Key']: t['Value'] for t in tags['TagList']}
        self.assertFalse('custodian_next' in tag_map)


class RDSClusterSnapshotTest(BaseTest):

    def test_rdscluster_snapshot_simple(self):
        session_factory = self.replay_flight_data(
            'test_rdscluster_snapshot_simple')
        p = self.load_policy({
            'name': 'rdscluster-snapshot-simple',
            'resource': 'rds-cluster-snapshot'},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)

    def test_rdscluster_snapshot_simple_filter(self):
        session_factory = self.replay_flight_data(
            'test_rdscluster_snapshot_simple')
        p = self.load_policy({
            'name': 'rdscluster-snapshot-simple-filter',
            'resource': 'rds-cluster-snapshot',
            'filters': [
                {'type': 'value',
                 'key': 'StorageEncrypted',
                 'value': False}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_rdscluster_snapshot_age_filter(self):
        factory = self.replay_flight_data('test_rdscluster_snapshot_simple')
        p = self.load_policy({
            'name': 'rdscluster-snapshot-age-filter',
            'resource': 'rds-cluster-snapshot',
            'filters': [{'type': 'age', 'days': 7}]},
            session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)

    def test_rdscluster_snapshot_trim(self):
        factory = self.replay_flight_data('test_rdscluster_snapshot_delete')
        p = self.load_policy({
            'name': 'rdscluster-snapshot-trim',
            'resource': 'rds-cluster-snapshot',
            'actions': ['delete']},
            session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)



# Copyright 2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

import json
from .common import BaseTest, functional
from botocore.exceptions import ClientError

class GlacierTagTest(BaseTest):

    @functional
    def test_glacier_tag(self):
        session_factory = self.replay_flight_data('test_glacier_tag')
        client = session_factory().client('glacier')
        name = 'c7n-glacier-test'

        client.create_vault(vaultName=name)
        self.addCleanup(client.delete_vault, vaultName=name)

        p = self.load_policy({
            'name': 'glacier',
            'resource': 'glacier',
            'filters': [
                {
                    'type': 'value',
                    'key': 'VaultName',
                    'value': 'c7n-glacier-test'
                }
            ],
            'actions': [
                {
                    'type': 'tag',
                    'key': 'abc',
                    'value': 'xyz'
                }
            ]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['VaultName'], name)

        tags = client.list_tags_for_vault(vaultName=resources[0]['VaultName'])
        self.assertEqual(len(tags['Tags']), 1)
        self.assertTrue('abc' in tags['Tags'])

    def test_glacier_untag(self):
        session_factory = self.replay_flight_data('test_glacier_untag')
        client = session_factory().client('glacier')

        p = self.load_policy({
            'name': 'glacier',
            'resource': 'glacier',
            'filters': [
                {
                    'tag:abc': 'present'
                }
            ],
            'actions': [
                {
                    'type': 'remove-tag',
                    'tags': ['abc']
                }
            ]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        tags = client.list_tags_for_vault(vaultName=resources[0]['VaultName'])
        self.assertEqual(len(tags['Tags']), 0)

    def test_glacier_markop(self):
        session_factory = self.replay_flight_data('test_glacier_markop')
        client = session_factory().client('glacier')
        name = 'c7n-glacier-test'

        p = self.load_policy({
            'name': 'glacier',
            'resource': 'glacier',
            'filters': [
                {
                    'tag:abc': 'present'
                }
            ],
            'actions': [
                {
                    'type': 'mark-for-op',
                    'op': 'notify',
                    'days': 4
                }
            ]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

        tags = client.list_tags_for_vault(vaultName=resources[0]['VaultName'])
        self.assertEqual(len(tags['Tags']), 2)
        self.assertTrue('maid_status' in tags['Tags'])


class GlacierStatementTest(BaseTest):

    @functional
    def test_glacier_remove_matched(self):
        session_factory = self.replay_flight_data('test_glacier_remove_matched')
        client = session_factory().client('glacier')
        name = 'test-glacier-remove-matched'
        client.create_vault(vaultName=name)
        self.addCleanup(client.delete_vault, vaultName=name)
        vault_arn = client.describe_vault(vaultName=name)['VaultARN']
        client.set_vault_access_policy(
            vaultName=name,
            policy={'Policy':json.dumps({
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "SpecificAllow",
                        "Effect": "Allow",
                        "Principal": {
                            "AWS": "arn:aws:iam::123456789012:root"
                        },
                        "Action": "glacier:AddTagsToVault",
                        "Resource": vault_arn
                    },
                    {
                        "Sid": "Public",
                        "Effect": "Allow",
                        "Principal": {
                            "AWS": "*"
                        },
                        "Action": "glacier:AddTagsToVault",
                        "Resource": vault_arn
                    }
                ]
            })})

        p = self.load_policy({
            'name': 'glacier-rm-matched',
            'resource': 'glacier',
            'filters': [
                {'VaultName': name},
                {'type': 'cross-account',
                 'whitelist': ["123456789012"]}],
            'actions': [
                {'type': 'remove-statements',
                 'statement_ids': 'matched'}]
            },
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual([r['VaultName'] for r in resources], [name])

        data = json.loads(
            client.get_vault_access_policy(
                vaultName=resources[0]['VaultName']).get('policy')['Policy'])
        self.assertEqual(
            [s['Sid'] for s in data.get('Statement', ())],
            ['SpecificAllow'])


    @functional
    def test_glacier_remove_named(self):
        session_factory = self.replay_flight_data('test_glacier_remove_named')
        client = session_factory().client('glacier')
        name = 'test-glacier-remove-named'

        client.create_vault(vaultName=name)
        self.addCleanup(client.delete_vault, vaultName=name)
        vault_arn = client.describe_vault(vaultName=name)['VaultARN']
        client.set_vault_access_policy(
            vaultName=name,
            policy={'Policy':json.dumps({
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "WhatIsIt",
                        "Effect": "Allow",
                        "Principal": "*",
                        "Action": ["glacier:DescribeVault"],
                        "Resource": vault_arn
                    }
                ]
            })}
        )

        p = self.load_policy({
            'name': 'glacier-rm-named',
            'resource': 'glacier',
            'filters': [{'VaultName': name}],
            'actions': [
                {'type': 'remove-statements',
                 'statement_ids': ['WhatIsIt']}]
            },
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertRaises(
            ClientError,
            client.get_vault_access_policy,
            vaultName=resources[0]['VaultName'])

    @functional
    def test_glacier_remove_statement(self):
        session_factory = self.replay_flight_data('test_glacier_remove_statement')
        client = session_factory().client('glacier')
        name = 'test-glacier-remove-statement'

        client.create_vault(vaultName=name)
        self.addCleanup(client.delete_vault, vaultName=name)
        vault_arn = client.describe_vault(vaultName=name)['VaultARN']
        client.set_vault_access_policy(
            vaultName=name,
            policy={'Policy':json.dumps({
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "SpecificAllow",
                        "Effect": "Allow",
                        "Principal": {
                            "AWS": "*"
                        },
                        "Action": "glacier:AddTagsToVault",
                        "Resource": vault_arn
                    },
                    {
                        "Sid": "RemoveMe",
                        "Effect": "Allow",
                        "Principal": "*",
                        "Action": ["glacier:DescribeVault"],
                        "Resource": vault_arn
                    }
                ]
            })}
        )

        p = self.load_policy({
            'name': 'glacier-rm-statement',
            'resource': 'glacier',
            'filters': [{'VaultName': name}],
            'actions': [
                {'type': 'remove-statements',
                 'statement_ids': ['RemoveMe']}]
            },
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)
        
        data = json.loads(
            client.get_vault_access_policy(
                vaultName=resources[0]['VaultName']).get('policy')['Policy'])
        self.assertTrue('RemoveMe' not in [s['Sid'] for s in data.get('Statement', ())])
    


# Copyright 2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

from .common import BaseTest, functional
from botocore.exceptions import ClientError
import json, time



class TestSqsAction(BaseTest):

    @functional
    def test_sqs_delete(self):
        session_factory = self.replay_flight_data(
            'test_sqs_delete')
        client = session_factory().client('sqs')
        client.create_queue(QueueName='test-sqs')
        queue_url = client.get_queue_url(QueueName='test-sqs')['QueueUrl']

        p = self.load_policy({
            'name': 'sqs-delete',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'delete'}]},
            session_factory=session_factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertRaises(
            ClientError,
            client.purge_queue, QueueUrl=queue_url)


    @functional
    def test_sqs_set_encryption(self):
        session_factory = self.replay_flight_data(
            'test_sqs_set_encryption')

        client_sqs = session_factory().client('sqs')
        client_sqs.create_queue(QueueName='sqs-test')
        queue_url = client_sqs.get_queue_url(QueueName='sqs-test')['QueueUrl']
        self.addCleanup(client_sqs.delete_queue, QueueUrl=queue_url)

        client_kms = session_factory().client('kms')
        key_id = client_kms.create_key(Description='West SQS encryption key')['KeyMetadata']['KeyId']
        client_kms.create_alias(
            AliasName='alias/new-key-test-sqs',
            TargetKeyId=key_id)
        self.addCleanup(client_kms.disable_key, KeyId=key_id)

        p = self.load_policy({
            'name': 'sqs-delete',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'set-encryption',
                 'key': 'new-key-test-sqs'}]},
            session_factory=session_factory)
        resources = p.run()

        check_master_key = client_sqs.get_queue_attributes(
            QueueUrl=queue_url,
            AttributeNames=['All'])['Attributes']['KmsMasterKeyId']
        self.assertEqual(check_master_key, 'c4816d44-73c3-4eed-a7cc-d52a74fa3294')


    @functional
    def test_sqs_remove_matched(self):
        session_factory = self.replay_flight_data('test_sqs_remove_matched')
        client = session_factory().client('sqs')
        name = 'test-sqs-remove-matched-1'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        client.set_queue_attributes(
            QueueUrl=queue_url,
            Attributes={'Policy':json.dumps({
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "SpecificAllow",
                        "Effect": "Allow",
                        "Principal": {
                            "AWS": "arn:aws:iam::123456789012:root"
                        },
                        "Action": [
                            "sqs:Subscribe"
                        ]
                    },
                    {
                        "Sid": "Public",
                        "Effect": "Allow",
                        "Principal": "*",
                        "Action": [
                            "sqs:GetqueueAttributes"
                        ]
                    }
                ]
            })}
        )

        p = self.load_policy({
            'name': 'sqs-rm-matched',
            'resource': 'sqs',
            'filters': [
                {'QueueUrl': queue_url},
                {'type': 'cross-account',
                 'whitelist': ["123456789012"]}
            ],
            'actions': [
                {'type': 'remove-statements',
                 'statement_ids': 'matched'}]
            },
            session_factory=session_factory)
        resources = p.run()

        self.assertEqual([r['QueueUrl'] for r in resources], [queue_url])

        data = json.loads(client.get_queue_attributes(QueueUrl=resources[0]['QueueUrl'], AttributeNames=['Policy'])['Attributes']['Policy'])
        self.assertEqual(
            [s['Sid'] for s in data.get('Statement', ())],
            ['SpecificAllow'])


    @functional
    def test_sqs_remove_named(self):
        session_factory = self.replay_flight_data('test_sqs_remove_named')
        client = session_factory().client('sqs')
        name = 'test-sqs-remove-named'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        client.set_queue_attributes(
            QueueUrl=queue_url,
            Attributes={'Policy':json.dumps({
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "SpecificAllow",
                        "Effect": "Allow",
                        "Principal": {
                            "AWS": "arn:aws:iam::644160558196:root"
                        },
                        "Action": ["sqs:Subscribe"]
                    },
                    {
                        "Sid": "RemoveMe",
                        "Effect": "Allow",
                        "Principal": "*",
                        "Action": ["sqs:GetqueueAttributes"]
                    }
                ]
            })}
        )

        p = self.load_policy({
            'name': 'sqs-rm-named',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'remove-statements',
                 'statement_ids': ['RemoveMe']}]
            },
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)

        data = json.loads(client.get_queue_attributes(QueueUrl=resources[0]['QueueUrl'], AttributeNames=['Policy'])['Attributes']['Policy'])
        self.assertTrue('RemoveMe' not in [s['Sid'] for s in data.get('Statement', ())])


    @functional
    def test_sqs_mark_for_op(self):
        session_factory = self.replay_flight_data('test_sqs_mark_for_op')
        client = session_factory().client('sqs')
        name = 'test-sqs'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        p = self.load_policy({
            'name': 'sqs-mark-for-op',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'mark-for-op',
                 'tag': 'tag-for-op',
                 'op': 'delete',
                 'days': 1}]
        },
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)
        tags_after_run = client.list_queue_tags(
            QueueUrl=queue_url).get('Tags', {})
        self.assertTrue("tag-for-op" in tags_after_run)

    @functional
    def test_sqs_tag(self):
        session_factory = self.replay_flight_data('test_sqs_tags')
        client = session_factory().client('sqs')
        name = 'test-sqs'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        p = self.load_policy({
            'name': 'sqs-mark-for-op',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'tag',
                 'key': 'tag-this-queue',
                 'value': 'This queue has been tagged'}]},
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)
        tags_after_run = client.list_queue_tags(
            QueueUrl=queue_url).get('Tags', {})
        self.assertTrue("tag-this-queue" in tags_after_run)

    @functional
    def test_sqs_remove_tag(self):
        session_factory = self.replay_flight_data('test_sqs_remove_tag')
        client = session_factory().client('sqs')
        name = 'test-sqs'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        client.tag_queue(
            QueueUrl=queue_url,
            Tags={
                'remove-this-tag': 'tag to be removed'
            })
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        p = self.load_policy({
            'name': 'sqs-mark-for-op',
            'resource': 'sqs',
            'filters': [{'QueueUrl': queue_url}],
            'actions': [
                {'type': 'remove-tag',
                 'tags': ['remove-this-tag']}]},
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)
        tags_after_run = client.list_queue_tags(
            QueueUrl=queue_url).get('Tags', {})
        self.assertTrue("remove-this-tag" not in tags_after_run)

    @functional
    def test_sqs_marked_for_op(self):
        session_factory = self.replay_flight_data('test_sqs_marked_for_op')
        client = session_factory().client('sqs')
        name = 'test-sqs'
        queue_url = client.create_queue(QueueName=name)['QueueUrl']
        client.tag_queue(
            QueueUrl=queue_url,
            Tags={
                'tag-for-op': 'Resource does not meet policy: delete@2017/11/01'
            })
        self.addCleanup(client.delete_queue, QueueUrl=queue_url)

        p = self.load_policy({
            'name': 'sqs-marked-for-op',
            'resource': 'sqs',
            'filters': [
                {'type': 'marked-for-op', 'tag': 'tag-for-op',
                 'op': 'delete'}]},
            session_factory=session_factory)

        resources = p.run()
        self.assertEqual(len(resources), 1)


# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

from .common import BaseTest


class TestOpsworksCM(BaseTest):

    def test_query_CM(self):
        factory = self.replay_flight_data('test_opswork-cm_query')
        p = self.load_policy({
            'name': 'get-opswork-cm',
            'resource': 'opswork-cm'
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['ServerName'], 'test-delete-opswork-cm')

    def test_delete_CM(self):
        factory = self.replay_flight_data('test_opswork-cm_delete')
        p = self.load_policy({
            'name': 'delete-opswork-cm',
            'resource': 'opswork-cm',
            'actions': ['delete']
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['ServerName'], 'test-delete-opswork-cm')
        client = factory().client('opsworkscm')
        remainder = client.describe_servers()['Servers']
        self.assertEqual(len(remainder), 1)
        self.assertEqual(remainder[0]['Status'], 'DELETING')


class TestOpsWorksStack(BaseTest):

    def test_query_opsworks_stacks(self):
        factory = self.replay_flight_data('test_opswork-stack_query')
        p = self.load_policy({
            'name': 'get-opswork-stack',
            'resource': 'opswork-stack'
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 2)
        self.assertEqual(
            sorted([r['Name'] for r in resources]),
            ['test-delete-opswork-stack', 'test-delete-opswork-stack2'])

    def test_stop_opsworks_stacks(self):
        factory = self.replay_flight_data('test_opswork-stack_stop')
        p = self.load_policy({
            'name': 'stop-opswork-stack',
            'resource': 'opswork-stack',
            'filters': [{'Name': 'test-delete-opswork-stack'}],
            'actions': ['stop']
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['Name'], 'test-delete-opswork-stack')
        client = factory().client('opsworks')
        remainder = client.describe_stack_summary(StackId=resources[0]['StackId'])['StackSummary']
        self.assertEqual(remainder['InstancesCount']['Stopping'], 1)

    def test_delete_opsworks_stacks(self):
        factory = self.replay_flight_data('test_opswork-stack_delete')
        p = self.load_policy({
            'name': 'delete-opswork-stack',
            'resource': 'opswork-stack',
            'filters': [{'Name': 'test-delete-opswork-stack'}],
            'actions': ['delete']
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['Name'], 'test-delete-opswork-stack')
        client = factory().client('opsworks')
        remainder = client.describe_stacks()['Stacks']
        self.assertEqual(len(remainder), 1)
        self.assertNotEqual(remainder[0]['Name'], 'test-delete-opswork-stack')
        


# Copyright 2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .common import BaseTest

class ReplInstance(BaseTest):

    def test_describe_augment_no_tags(self):
        session_factory = self.replay_flight_data(
            'test_dms_repl_instance_describe_sans_tags')
        p = self.load_policy({
            'name': 'dms-replinstance',
            'resource': 'dms-instance'},
            session_factory=session_factory)        
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['ReplicationInstanceIdentifier'],
                         'replication-instance-1')

    def test_describe_get_resources(self):
        session_factory = self.replay_flight_data(
            'test_dms_repl_instance_delete')
        p = self.load_policy({
            'name': 'dms-replinstance',
            'resource': 'dms-instance'},
            session_factory=session_factory)        
        resources = p.resource_manager.get_resources(
            ['replication-instance-1'])
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['ReplicationInstanceIdentifier'],
                         'replication-instance-1')        

    def test_delete(self):
        session_factory = self.replay_flight_data(
            'test_dms_repl_instance_delete')
        client = session_factory().client('dms')
        p = self.load_policy({
            'name': 'dms-replinstance',
            'resource': 'dms-instance',
            'actions': ['delete']},
            session_factory=session_factory)        
        resources = p.run()
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]['ReplicationInstanceIdentifier'],
                         'replication-instance-1')
        instances = client.describe_replication_instances().get(
            'ReplicationInstances')
        self.assertEqual(instances[0]['ReplicationInstanceStatus'], 'deleting')
        
        




# Copyright 2016-2017 Capital One Services, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import, division, print_function, unicode_literals

from .common import BaseTest


class TestAMI(BaseTest):

    def test_query(self):
        factory = self.replay_flight_data('test_ami')
        p = self.load_policy({
            'name': 'test-ami',
            'resource': 'ami',
            'filters': [
                {'Name': 'LambdaCompiler'},
                {'type': 'image-age', 'days': 0.2}],
            'actions': ['deregister']
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_unused_ami_true(self):
        factory = self.replay_flight_data('test_unused_ami_true')
        p = self.load_policy({
            'name': 'test-unused-ami',
            'resource': 'ami',
            'filters': ['unused']
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)

    def test_unused_ami_false(self):
        factory = self.replay_flight_data('test_unused_ami_false')
        p = self.load_policy({
            'name': 'test-unused-ami',
            'resource': 'ami',
            'filters': [{
                'type': 'unused','value': False}]
        }, session_factory=factory)
        resources = p.run()
        self.assertEqual(len(resources), 1)
